//------------------------------------------------------------------------------------//
//----------[   Surface coating macro by: [GDS|Entropy] aka Ian McDonald   ]----------//   
//----------[   Coating Implementation: Snow and Ice                       ]----------//
//----------[   ALPHA 1 release: Code date 12/27/2010                      ]----------//
//------------------------------------------------------------------------------------//

//  Begin Macros by Chris Huff
#macro RRand(RS, Min, Max) (rand(RS)*(Max-Min) + Min) #end  
//  End  

//------
//  randVect Macro by [GDS|Entropy]
//------

#macro randVector(Array,ctr,Min,Max,Method)
 #local i=0;
 #while (i<ctr)
  #switch (Method)
   #case (0)    
    #declare Array[i] = <RRand(RsA, Min, Max),RRand(RsA, Min, Max),RRand(RsA, Min, Max)>;
   #break
   #case (1)
    #declare Array[i] = <sin(RRand(RsA, Min, Max)),RRand(RsA, Min, Max),tan(RRand(RsA, Min, Max))>;
   #break    
   #case (2)
    #declare Array[i] = <sin(RRand(RsA, Min, Max)),atanh(RRand(RsA, Min, Max)),tan(RRand(RsA, Min, Max))>;
   #break    
   #case (3)
    #declare Array[i] = <asin(RRand(RsA, Min, Max)),atanh(RRand(RsA, Min, Max)),tan(RRand(RsA, Min, Max))>;
   #break
   #case (4)
    #declare Array[i] = <acosh(RRand(RsA, Min, Max)),asinh(RRand(RsA, Min, Max)),tan(RRand(RsA, Min, Max))>;
   #break
   #case (5)
    #declare Array[i] = <cos(i),i,sin(i)>;
   #break   
  #end  
 #local i=i+1;
 #end
#end

//------
//  randVectorMM Macro by [GDS|Entropy]
//------

#macro randVectorMM(Array,ctr,Min,Max,Method,Object)
 #local vectorA = min_extent(Object);
 #local vectorB = max_extent(Object); 
 #local i=0;
 #while (i<ctr)
  #switch (Method)
   #case (0)    
    #declare Array[i] = <RRand(RsA, Min+vectorA.x, Max+vectorB.x),RRand(RsA, Min+vectorA.y, Max+vectorB.y),RRand(RsA, Min+vectorA.z, Max+vectorB.z)>;
   #break
   #case (1)
    #declare Array[i] = <sin(RRand(RsA, Min+vectorA.x, Max+vectorB.x)),RRand(RsA, Min+vectorA.y, Max+vectorB.y),tan(RRand(RsA, Min+vectorA.z, Max+vectorB.z))>;
   #break    
   #case (2)
    #declare Array[i] = <sin(RRand(RsA, Min+vectorA.x, Max+vectorB.x)),atanh(RRand(RsA, Min+vectorA.y, Max+vectorB.y)),tan(RRand(RsA, Min+vectorA.z, Max+vectorB.z))>;
   #break    
   #case (3)
    #declare Array[i] = <asin(RRand(RsA, Min+vectorA.x, Max+vectorB.x)),atanh(RRand(RsA, Min+vectorA.y, Max+vectorB.y)),tan(RRand(RsA, Min+vectorA.z, Max+vectorB.z))>;
   #break
   #case (4)
    #declare Array[i] = <acosh(RRand(RsA, Min+vectorA.x, Max+vectorB.x)),asinh(RRand(RsA, Min+vectorA.y, Max+vectorB.y)),tan(RRand(RsA, Min+vectorA.z, Max+vectorB.z))>;
   #break
   #case (5)
    #declare Array[i] = <cos(i+RRand(RsA, Min+vectorA.x, Max+vectorB.x)),i+RRand(RsA, Min+vectorA.y, Max+vectorB.y),sin(i+RRand(RsA, Min+vectorA.z, Max+vectorB.z))>;
   #break  
  #end  
 #local i=i+1;
 #end
#end

//------

//  SnowIce Macro by [GDS|Entropy]
#macro SnowIce(ctr,RsA,vRangeMin,vRangeMax,vGenType,vGenMethod,testVector,testObject,radVal,strVal,useTexture,useInterior,useSnow,icicleSegs,icicleProbability,icicleTexture,snowTexture,iceInterior,upcastCount)
 #if (useSnow = false & icicleProbability <= 0)
  #error "You must specify as enabled Snow, Ice or both."
 #end
 
 #local vectorArray = array [ctr];
 
 #if (vGenType = 0)
  randVector(vectorArray,ctr,vRangeMin,vRangeMax,vGenMethod)
 #else
  randVectorMM(vectorArray,ctr,vRangeMin,vRangeMax,vGenMethod,testObject)
 #end
 
 #local vectorA = min_extent(testObject);
 #local vectorB = max_extent(testObject);
 
 #local testColorMethod = 3;  //    defaults to the specified method if textures disabled; 3 = off
 
 #declare snowCap = 
 blob { 
 #local i = 0;
 #while (i < ctr)     
   #if (inside(testObject,vectorArray[i]))
    #if (useSnow = true)
    //  Begin Snow block
    #local vecType = 5;
    #local vecCnt = 0;  //  you can override the orbiting by changing vecType and vecCnt
    #local wiggle = RRand(RsA, 2 - i,5 + i);  
    // the following section will change by alpha 2
    #local tVect0 = trace(testObject,vectorArray[i],<testVector.x + wiggle,testVector.y,testVector.z + wiggle>);
    #local tVect1 = trace(testObject,vectorArray[i],<testVector.x + wiggle,testVector.y,testVector.z - wiggle>);
    #local tVect2 = trace(testObject,vectorArray[i],<testVector.x - wiggle,testVector.y,testVector.z + wiggle>);
    #local tVect3 = trace(testObject,vectorArray[i],<testVector.x - wiggle,testVector.y,testVector.z - wiggle>);
    #local tVect4 = trace(testObject,vectorArray[i],testVector); 

    #while (vecCnt <= vecType)
     #switch (vecCnt)
      #case (0)    
       #local vActive = tVect0;
       //#debug "vActive = tVect0 \n"       
      #break 
      #case (1)    
       #local vActive = tVect1;
       //#debug "vActive = tVect1 \n"       
      #break 
      #case (2)    
       #local vActive = tVect2;
       //#debug "vActive = tVect2 \n"       
      #break 
      #case (3)    
       #local vActive = tVect3;
       //#debug "vActive = tVect3 \n"
      #break 
      #case (4)    
       #local vActive = tVect4;
       //#debug "vActive = tVect4 \n"
      #break                                  
     #end 
     
     // end area of change
     
     sphere {
      vActive, radVal-RRand(RsA, 0.125, 0.25), strVal     
      #if (useTexture)
       texture {
        snowTexture
       }
      #else
       #if (testColorMethod = 0)      
        pigment {color rgb vectorArray[i]}
       #end 
       #if (testColorMethod = 1)      
        pigment {color rgb <1,1,1>}
       #end
       #if (testColorMethod = 2)      
        pigment {color rgb <1,0,0>}
       #end             
      #end
     }
     #local vecCnt = vecCnt + 1;
    #end
    #end 
    //  End Snow block
    
    //  Begin Icicle block: Derivative work based upon Jaime Vives Piqueres "Stalactite" code
    #if (RRand(RsA, 0, 2) <= (icicleProbability * 0.01))  
    //#debug "Icicle Formed! \n"    
    #local multiplier = 2;
    #local foo = 0.1;
    #local bar = rand(RsA);
    #local inum = 200 * bar;   
    #local rad = (foo * 0.4) + (foo * radVal * bar * multiplier);
    #local ictr = 1;
   
    #while (ictr <= inum)
     #local irad = rad * (1 - sin(ictr / inum));
     #local iwiggle = (ictr / inum) * 0.4 * multiplier;
       
     #if (rand(RsA) > 0.5)
      #local xwiggle = RRand(RsA, 1, 1.5);
     #else
      #local xwiggle = RRand(RsA, -1, -1.5);
     #end 
    
     #if (rand(RsA) > 0.5)
      #local zwiggle = RRand(RsA, 1, 1.5);
     #else
      #local zwiggle = RRand(RsA, -1, -1.5);
     #end
       
     #local iVect = <xwiggle * irad * rand(RsA) * (0.5 - (0.4 * (ictr / inum))), -iwiggle * 2, zwiggle * irad * rand(RsA) * (0.5 - (0.4 * (ictr / inum)))>;
     
     //  The icicle     
     sphere {
      trace(testObject,<vectorArray[i].x, vectorArray[i].y, vectorArray[i].z>,-testVector), irad * 2,1
      translate iVect 
      translate y * (iVect.y * 1.5)     
      #if (useTexture)
       texture {
        icicleTexture
       }
      #else      
       #if (testColorMethod = 0)      
        pigment {color rgb vectorArray[i]}
       #end 
       #if (testColorMethod = 1)      
        pigment {color rgb <1,1,1>}
       #end
       #if (testColorMethod = 2)      
        pigment {color rgb <0,1,0>}
       #end
      #end            
     } 
     
     // Upcast ice, connecting icicles and main coating of ice/snow on object   
     #local ui = 0;
     #while (ui < upcastCount)                                                                                                                       
      #local sVect = <vectorArray[i].x, -vectorArray[i].y, vectorArray[i].z>;
      #local sTrace = trace(testObject,sVect,<-sin(iVect.x + ui + RRand(RsA, 0, 1)), iVect.y, -cos(iVect.z + ui + RRand(RsA, 0, 1))>);
      #local sDist = vlength(sTrace - iVect);
      #local sRad = abs((irad * 2) - log(sDist));     
    
      #if ((sRad < (irad * 3)) & vlength(sTrace) != 0)
       sphere {
        <sTrace.x, sTrace.y, sTrace.z>, sRad,1
        #if (useTexture)
         texture {
          icicleTexture
         }
        #else      
         #if (testColorMethod = 0)      
          pigment {color rgb vectorArray[i]}
         #end 
         #if (testColorMethod = 1)      
          pigment {color rgb <1,1,1>}
         #end 
         #if (testColorMethod = 2)      
          pigment {color rgb <0,0,1>}
         #end          
        #end            
       }     
      #end
    
      #local ui = ui + 1; 
     #end
         
    #local ictr = ictr + 1;
   #end 
    
   #end
   //  End Icicle block
            
   #end                                            
 #local i = i + 1;
 #end
 #if (useInterior)
  interior { 
   iceInterior 
  }
 #end 
 }

object {
 snowCap
}

#end
//------