//      by [GDS|Entropy]
#include "SnowIce.inc"

global_settings {
 max_trace_level 256
}

camera {
 location <-5, 6,-17>
 up y*image_height right x*image_width
 look_at < 0, 1.5, 0>
 angle 45
} 

light_source {
  0*x                  
  color rgb <1,1,1>    
  translate <-20, 40, -20>
}


#declare cyl =
cylinder {
 <-2,0,0>,<2,0,0>, 1
 translate <0,0,0>
 scale 2 
} 

#declare sph = 
sphere {
 <0,0,0>, 2
 translate <0,0,0>
 scale 1
}

#declare sphcyl = 
merge {
 object {
  sph
  translate <0,0,-1>
 }
 object {
  cyl
 }
}

#declare planeo =
plane {
 y, 0
 texture {
  pigment {
   color rgb 1   
  } 
  finish {
   diffuse 0.65
   ambient 0
  }
 } 
 translate y * -5
}

#declare tor =
torus { 
 1, 0.5 
 translate <0,0,0>
 rotate <90,-45,0>
 scale 2
}

#declare bifurcations = 2;
#declare rseed = seed(27182818);

#macro treeX(origin,scaleFactor,numIter)
union {
 #if(numIter > 0)
  union {
    cylinder {
     <0,0,0>, <0,1,0>,  0.05 
     scale scaleFactor
    }     
   #local J = bifurcations*(1 + rand(rseed))/2;
   #while(J > 0 )
    object {
     treeX(origin,1,numIter-1)
     rotate x*(25 + rand(rseed)*10)
     rotate y*(J+numIter)*360/bifurcations
     scale scaleFactor
     translate y
    }
    #local J = J - 1;
   #end
  } 
 #else
  cylinder {
   <0,0,0>, <0,1,0>,  0.05 
   scale scaleFactor
  }
 #end
}
#end

#declare tree =
object {
 treeX(<0,0,0>,3,3)
 texture {
  pigment {
   color rgb 1   
  } 
  finish {
   diffuse 0.65
   ambient 0
  }
 } 
}

#declare sceneEl0 =
union {
 merge {
  object {
   sphcyl
   translate <2,0,0>
  }
  object {
   tor
   translate <-2.5,0,-2>
  }
 }
 object {
  planeo
 }
}

#declare sceneEl1 =
 object {
  tree
  scale 1
  translate <3,-5,-3>
 }

#declare filterVal = 0.85;

#declare Ice2 = 
texture {
 pigment {
  bozo 
  octaves 1 
  frequency 4 
  triangle_wave
  color_map { 
   [0.00 rgbt <0.97,0.97,0.98,filterVal+0.1>]
   [0.03 rgbt <0.91,0.97,0.97,filterVal+0.1>]
   [0.40 rgbt <0.91,0.91,0.93,filterVal+0.1>]
   [0.41 rgbt <0.91,0.94,0.94,filterVal+0.1>]
   [0.42 rgbt <0.91,0.91,0.97,filterVal+0.1>]
   [1.00 rgbt <0.91,0.91,0.97,filterVal+0.1>]
  }
 }
 finish { 
  ambient 0 
  diffuse 1 
  brilliance 2 
  specular 0.5 
  roughness 0.001 
  reflection {
   0.05, 0.25
  }          
  conserve_energy
 }
}

#declare iceInterior =
  interior {
   fade_distance 2
   fade_power 2
   ior 1.309 // ior of ice
   caustics 1.0 
  } 

//------[Macro Definition]------//
//#macro SnowIce(ctr,RsA,vRangeMin,vRangeMax,vGenType,vGenMethod,testVector,testObject,radVal,strVal,useTexture,useInterior,icicleSegs,icicleProbability,icicleTexture,snowTexture,iceInterior,upcastCount)
//------[Macro Definition]------// 

//----------[   Macro Usage: Example Call       ]----------//
#declare targetObj0 = sceneEl0;

#declare st0 = 
SnowIce(
    30000,          //  Base number of particles; should match the size of the random vector array.
    seed(574647),   //  Random seed.
    -8,             //  Random vector minimum range; based on object size.
    8,              //  Random vector maximum range; based on object size.
    0,              //  Random vector generator type. (0,1) 
    0,              //  Random vector generator method (0-5)
    <0,12,0>,       //  Base vector used by Trace()
    targetObj0,     //  The object to encrust.
    0.75,           //  Base Blob radius value.
    1,              //  Base Blob strength value.
    false,          //  Use Texture?
    false,          //  Use Interior?
    true,           //  Use Snow?
    12,             //  Number of Icicle segments.
    10,             //  Base probability of Icicle occurance. 0 = off
    Ice2,           //  Icicle Texture.
    Ice2,           //  Snow Texture.
    iceInterior,    //  The interior to apply.
    30              //  Base upcast particle count.
    );

//  NOTE:
//  If you choose to apply texture and interior to snow/ice individually, 
//  expect *very* long render times! Otherwise, apply to entire object.
//  You *can* call the macro twice, once on the target object with snow
//  off and icicles on, then once on the union of the target object and
//  the output of the macro with icicles off. This way you can use 
//  different textures for the ice and the snow.

//----------[   Macro Usage: End Example Call   ]----------//

#declare snowyObject0 =
union {
 object { 
  st0   
  texture {
   Ice2
  }
  interior {
   iceInterior
  }  
 }     
 object {
  targetObj0
  texture { 
   pigment {
    color rgb <145/255,102/255,32/255>
   }  
  }
  finish { 
   ambient 0 
   diffuse 1 
   specular 0.5 
   roughness 0.01 
   reflection 0.03
  }
  normal { 
   wrinkles 0.2 
   scale 0.5 
  }       
 }                
 translate y * 3
 rotate y * 45
}

object {
 snowyObject0
}